"""
TestVariableLoader class definition
"""

import unittest
from sodapclient.VariableLoader import VariableLoader
import numpy


class TestVariableLoader(unittest.TestCase):

    """
    Test class for VariableLoader class.
    """

    def setUp(self):

        self.url = 'http://myserver.org/mydata'
        # NB: Handler would have removed .html if present
        self.dataset_name = 'TestDataset'
        self.dds = {'lat': ['Float32', [10], ['lat']],
                    'long': ['Float32', [20], ['long']],
                    'depth': ['Int16', [10, 20], ['lat', 'long']]
                    }
        self.depth = numpy.ndarray(shape=[10, 20], dtype='int32')
        # Using int32 for DDS Int16 (see Definitions.py)
        cind = [icl for icl in range(self.depth.shape[1])]
        for rind in range(self.depth.shape[0]):
            self.depth[rind] = cind
            self.depth[rind] *= rind

        self.dim_sels = numpy.ndarray(shape=(2, 3), dtype='int')
        self.dim_sels[0, :] = [0, 4, 9]
        self.dim_sels[1, :] = [0, 2, 19]

        self.depth_sel = self.depth[self.dim_sels[0, 0]: self.dim_sels[0, 2]:
                                    self.dim_sels[0, 1],
                                    self.dim_sels[1, 0]: self.dim_sels[1, 2]:
                                    self.dim_sels[1, 1]]

        self.var_data = '...blah ... xx Int16 depth[lat = 3][long = 10] xx \
        ..Data:\nxxxxyyyy'.encode('utf-8') + self.depth_sel.tobytes()

        self.byte_ord_str = '<'

    def tearDown(self):
        pass

    def test_constructor(self):

        """
        Test the VariableLoader constructor.
        """

        var_loader = VariableLoader(self.url, self.dataset_name, self.dds)
        return var_loader

    def test_variable_name_validity(self):

        """
        Test variable name.
        """

        var_loader = self.test_constructor()
        requrl = var_loader.get_request_url('height', [])
        self.assertEqual(requrl, '')

    def test_number_of_dims(self):

        """
        Test number of dimensions.
        """

        var_loader = self.test_constructor()
        dim_sels = numpy.ndarray(shape=(1, 3), dtype='int32')
        requrl = var_loader.get_request_url('depth', dim_sels)
        self.assertEqual(requrl, '')

    def test_dim_selection(self):

        """
        Test dimension selection.
        """

        var_loader = self.test_constructor()
        dim_sels = self.dim_sels
        dim_sels[1, 2] = 20
        requrl = var_loader.get_request_url('depth', dim_sels)
        self.assertEqual(requrl, '')

    def test_url_construction(self):

        """
        Test URL construction.
        """

        var_loader = self.test_constructor()
        requrl = var_loader.get_request_url('depth', self.dim_sels)
        self.assertEqual(requrl, self.url + '.dods?depth%5B0:4:9%5D%5B0:2:19%5D')

    def test_load_variable(self):

        """
        Test variable loading.
        """

        var_loader = self.test_constructor()
        var = var_loader.load_variable('depth', self.var_data, self.dim_sels,
                                       self.byte_ord_str)
        self.assertTrue(numpy.array_equal(var, self.depth_sel))


if __name__ == "__main__":
    unittest.main()



"""
Handler class definition
"""

import urllib.request as ureq
from urllib.parse import urlparse as upar

from datetime import datetime

import numpy

from sodapclient.ProxyDict import ProxyDict
from sodapclient.DDSParser import DDSParser
from sodapclient.DASParser import DASParser
from sodapclient.VariableLoader import VariableLoader


class Handler:

    """
    The main sodapclient class.

    Both atomic variables and constructor variables are handled.
    Note that map vectors within constructors are treated
    as atomic variables. This means that existing variable names will be
    overwritten whenever they are repeated (whether atomic variables or
    map vectors within a constructor). E.g. if a Grid contains an atomic
    N-dimensional array it will be included in the returned dds and the
    variable will be loaded but any map vectors with the same name as an
    existing atomic variable will overwrite the existing variable.
    Therefore the implementation will work provided map vectors defined within
    a constructor are identical to any variable with the same name defined
    outside the constructor. This should not be a limitation as all OpenDAP
    datasets seem to adhere to this anyway.

    args...
        url: OPeNDAP URL
    kwargs...
        proxy_file_name: Name of text file containing proxy details
        credentials: Credentials dictionary
        log: Creates log file on True
    """

    def __init__(self, url: str, proxy_file_name: str=None, credentials: dict=None, log: bool=False) -> None:

        """
        args...
            url: URL
        kwargs...
            proxy_file_name: path to text file containing proxy details
            log: whether to create a log file
        """

        self.log_file = None
        if log:
            log_file_name = '/var/tmp/sodapclient-log-' + \
                str(datetime.now().timestamp()) + '.txt'
            self.log_file = open(log_file_name, 'wt')
            self.log_file.write('sodapclient created at ' +
                                str(datetime.now()) + '\n\n')

        # Set up the proxy (if required)
        if proxy_file_name is not None:
            self.proxy_dict = ProxyDict(proxy_file_name)
            if self.proxy_dict.valid_proxy:
                self.set_up_proxy()
        if self.log_file:
            if self.proxy_dict.valid_proxy:
                self.log_file.write('Using Proxy Server.\n')
            else:
                self.log_file.write('Invalid Proxy.\n')

        # Set up authentication (if required)
        if credentials is not None:
            self.credentials = credentials
            self.set_credentials()

        # Check the URL is valid
        parts = upar(url)
        if not all(parts[:3]):
            msg = 'ERROR: Invalid URL format\n'
            if self.log_file:
                self.log_file.write(msg)
            raise ValueError(msg)

        self.base_url = url
        if self.base_url[-4:] == 'html':   # Remove .html extension if present
            self.base_url = self.base_url[:-5]

        if self.log_file:
            self.log_file.write('Base URL: ' + self.base_url + '\n')

        # Set up the dds
        self.get_dds()

        # Set up the das
        self.get_das()

        # Set up the (initially empty) variables dictionary
        self.variables = {}  # Dictionary to hold loaded variables

    def __del__(self) -> None:

        """
        Close the log file when the handler is deleted.
        """

        if self.log_file:
            self.log_file.write('\nHandler destroyed at ' +
                                str(datetime.now()) + '\n\n')
            self.log_file.close()

    def set_up_proxy(self) -> None:

        """
        Set up the proxy.
        """

        proxy_handler = ureq.ProxyHandler(self.proxy_dict.get_dict())
        opener = ureq.build_opener(proxy_handler)
        ureq.install_opener(opener)

    def set_credentials(self) -> None:

        """
        Set up credentials.
        """

        pswd_mgr = ureq.HTTPPasswordMgr()
        pswd_mgr.add_password(realm=self.credentials['realm'], uri=self.credentials['top-level-url'],
                              user=self.credentials['username'], passwd=self.credentials['password'])
        auth_handler = ureq.HTTPBasicAuthHandler(pswd_mgr)
        opener = ureq.build_opener(auth_handler)
        ureq.install_opener(opener)

    def get_dds(self) -> None:

        """
        Get the Dataset Descriptor Structure (DDS).
        """

        turl = self.base_url + '.dds'
        try:
            urlo = ureq.urlopen(turl)
        except ureq.HTTPError:
            self.dds = None
            return
        dds_str = urlo.read().decode('utf-8')
        urlo.close()
        dds_parser = DDSParser()
        dds_parser.parse(dds_str)
        self.dataset_name = dds_parser.dataset_name
        self.dds = dds_parser.data
        if self.log_file:
            self.log_file.write('\n--- dds ---\n\n')
            dds_parser.print_dds_to_file(self.log_file)

    def get_das(self) -> None:

        """
        Get the Dataset Attribute Structure (DAS).
        """

        turl = self.base_url + '.das'
        try:
            urlo = ureq.urlopen(turl)
        except ureq.HTTPError:
            self.das = None
            return
        das_str = urlo.read().decode('utf-8')
        urlo.close()
        das_parser = DASParser()
        das_parser.parse(das_str)
        self.das = das_parser.data
        if self.log_file:
            self.log_file.write('--- das ---\n\n')
            das_parser.print_das_to_file(self.log_file, das=None)
            self.log_file.write('-----------\n\n')

    def get_variable(self, var_name: str, dim_sels: numpy.ndarray, byte_ord_str: str, check_type: bool=True) -> None:

        """
        Get a variable.
        args...
            var_name: variable name
            dim_sels: dimension selections (see VariableLoader.get_request_url)
            byte_ord_str: byte order string: '<' for little endian, '>' for big endian
        kwargs...
            check_type: check header string contains expected variable type
        """

        var_loader = VariableLoader(self.base_url, self.dataset_name, self.dds)
        requrl = var_loader.get_request_url(var_name, dim_sels)
        if requrl is not None:
            with ureq.urlopen(requrl) as urlo:
                var_data = urlo.read()
            var = var_loader.load_variable(var_name, var_data, dim_sels,
                                           byte_ord_str, check_type)
            if var is not None:
                self.variables[var_name] = var

    def print_status(self) -> None:

        """
        Print the handler status to the console.
        """

        print('HANDLER STATUS...\n')
        print('Base URL:', self.base_url, '\n')
        print('dds:\n')
        psr = DDSParser()  # Temporary object - just need printing function
        psr.print_dds(self.dataset_name, self.dds)
        print('das:\n')
        psr = DASParser()  # Temporary object - just need printing function
        psr.print_das(self.das)  # Convenience instantiation just to print dds

    def print(self) -> None:

        """
        Print the handler status and available variables.
        """

        self.print_status()
        print('VARIABLES...\n')
        if len(self.variables) > 0:
            print('Variables loaded:\n')
            for var in self.variables.keys():
                print('Variable name :', var)
                print('Loaded dimensions :', self.variables[var].shape)
                print('data...')
                print(self.variables[var])
        else:
            print('No variables loaded.')



"""
Parser class definition
"""

from io import TextIOWrapper

class Parser:

    """
    Serves as base class for DDSParser and DASParser classes.
    """

    def __init__(self) -> None:

        self.dtype = 'Undefined'  # Name of parsed dataset (e.g. DDS, DAS)
        self.data = {}  # data dictionary
        self.indts = []  # Indentation list
        self.data_lines = []  # List of lines in data to be parsed
        self.lnum = 0  # Current line number
        self.finished = False  # Flag to indicate parsing complete

    def find_start(self, data_str: str, start_str: str) -> None:

        """
        Find the start of a dataset.
        args...
            data_str: string to be searched
            start_str: string indicating start of dataset
        """

        # Remove any empty lines
        temp_lines = data_str.split('\n')
        for l in temp_lines:
            if len(l) > 0:
                self.data_lines.append(l)

        # Loop to find start of dataset
        while start_str not in self.data_lines[self.lnum]:
            if self.lnum == len(self.data_lines) - 1:
                print('Parser: No datasets found, stopping.')
                break
            self.lnum += 1

        # Get root indentation level and store in list
        if self.lnum < len(self.data_lines) - 1:
            self.indts.append(self.find_indent_level
                              (self.data_lines[self.lnum]))

    @staticmethod
    def find_indent_level(line: str) -> int:

        """
        Find the indentation level.
        args:
            line: line to find indentation level
        """

        i = 0
        if len(line) > 0:
            el = line.split()[0]  # First element
            while line[i:i + len(el)] != el:
                i += 1
        return i

    def check_line(self) -> bool:

        """
        Check whether to process a line.
        """

        do_line = True

        self.lnum += 1
        indt = self.find_indent_level(self.data_lines[self.lnum])

        if indt > self.indts[-1]:  # Increased: expecting new variable
            self.indts.append(indt)
        elif indt < self.indts[-1]:  # Decreased: end of current body
            while (len(self.indts) > 0) & (indt < self.indts[-1]):
                self.indts.pop()
            do_line = False
            if len(self.indts) == 1:  # Back to root level so must be finished
                self.finished = True

        return do_line

    @staticmethod
    def print_data(dtype: str, data: dict) -> None:

        """
        Print data type and data (these can be passed in externally for convenience).
        args:
            dtype: data type
            data: data to be printed
        """

        print('dtype :', dtype, '\n')
        if len(data) == 0:
            print('Structure is not defined')

    @staticmethod
    def print_data_to_file(dtype: str, data: dict, file: TextIOWrapper) -> None:

        """
        Print data type and data to a file (these can be passed in externally for convenience).
        args:
            dtype: data type
            data: data to be printed
            file: file object
        """

        file.write('dtype :' + dtype + '\n\n')
        if len(data) == 0:
            file.write('Structure is not defined\n')



"""
TestDDSParser class definition
"""

import os
import unittest
from sodapclient.DDSParser import DDSParser


class TestDDSParser(unittest.TestCase):

    """
    Test class for DDSParser class.
    """

    def setUp(self):

        # Create the test DDS string
        self.dds_str = """
Dataset {
    Float32 lat[lat = 10];
    Float32 long[long = 20];
    Grid {
     ARRAY:
       Int16 depth[lat = 10][long = 20];
     MAPS:
       Float32 lat[lat = 10];
       Float32 long[long = 20];
    } depth;
} TestDataset;
"""

        self.dataset_name = 'TestDataset'
        self.dds = {'lat': ['Float32', [10], ['lat']],
                    'long': ['Float32', [20], ['long']],
                    'depth': ['Int16', [10, 20], ['lat', 'long']]
                    }

        self.test_file_name = 'dasparser_out.txt'

    def tearDown(self):
        if os.path.exists(self.test_file_name):
            os.remove(self.test_file_name)

    def test_constructor(self):

        """
        Test the DDSParser constructor.
        """

        parser = DDSParser()
        self.assertTrue(type(parser) == DDSParser)

    def test_parse(self):

        """
        Test the parse method.
        """

        dds_parser = DDSParser()
        dds_parser.parse(self.dds_str)
        self.assertEqual(self.dataset_name, dds_parser.dataset_name)
        self.assertEqual(self.dds, dds_parser.data)

    def test_print(self):

        """
        Test the print method.
        """

        dds_parser = DDSParser()
        dds_parser.parse(self.dds_str)
        dds_parser.print_dds()

    def test_print_to_file(self):

        """
        Test the print to file method.
        """

        dds_parser = DDSParser()
        dds_parser.parse(self.dds_str)
        with open(self.test_file_name, 'wt') as test_file:
            dds_parser.print_dds_to_file(test_file)
        self.assertEqual(os.path.exists(self.test_file_name), True)


if __name__ == "__main__":
    unittest.main()



"""
VariableLoader class definition
"""

import numpy
from sodapclient.Definitions import Definitions


class VariableLoader:

    """
    Variable loader class. Does some checks and returns the requested variable
    as a NumPy array.
    """

    def __init__(self, url: str, dataset_name: str, dds: dict) -> None:

        """
        args...
            url: URL
            dataset_name: dataset name (string)
            dds: DDS (dictionary)
        """

        # Get available DDS definitions
        self.url = url
        self.dataset_name = dataset_name
        self.dds = dds

    def get_request_url(self, var_name: str, dim_sels: numpy.ndarray) -> str:

        """
        Returns the URL for a request.
        args...
            var_name: variable name
            dim_sels: dimension selections variable: a numpy ndarray, type int, size N by 3.
                      Each row corresponds to one variable dimension, with columns containing
                      the min, step and max indexes.
        """

        # Check variable exists and dimensions are not exceeded

        if var_name not in self.dds:
            print('VariableLoader: Requested variable not in DDS, stopping.')
            return ''

        var_dims = self.dds[var_name][1]
        num_dims = len(var_dims)
        if (num_dims > 0) and (dim_sels.shape[0] != num_dims):
            print('VariableLoader: Requested number of dimensions incorrect, stopping.')
            return ''

        dims_ok = self.check_dim_sels(var_dims, dim_sels, num_dims)
        if not dims_ok:
            print('VariableLoader: At least one dimension selection request is not valid, stopping.')
            return ''

        # Construct the request url (replace square brackets with hex codes as needed by some servers)

        requrl = self.url + '.dods?' + var_name
        for idim in range(num_dims):
            dimstr = '%5B' + str(dim_sels[idim, 0]) + ':' + \
                     str(dim_sels[idim, 1]) + ':' + \
                     str(dim_sels[idim, 2]) + '%5D'
            requrl += dimstr

        return requrl

    @staticmethod
    def check_dim_sels(var_dims: list, dim_sels: numpy.ndarray, num_dims: int) -> bool:

        """
        Extract dimension selections and check they're valid.
        args...
            var_dims: variable dimensions
            dim_sels: dimension selections (see get_request_url)
            num_dims: number of dimensions
        """

        dims_ok = True
        for idim in range(num_dims):
            if (dim_sels[idim, 0] < 0) or (dim_sels[idim, 1] < 0) or \
               (dim_sels[idim, 2] < 0):
                dims_ok = False
            if (dim_sels[idim, 0] > var_dims[idim] - 1) or \
               (dim_sels[idim, 2] > var_dims[idim] - 1):
                dims_ok = False
            if dim_sels[idim, 2] < dim_sels[idim, 0]:
                dims_ok = False
            if (dim_sels[idim, 0] != dim_sels[idim, 2]) and \
               (dim_sels[idim, 1] > dim_sels[idim, 2]):
                dims_ok = False

        return dims_ok

    def load_variable(self, var_name: str, var_data: bytes, dim_sels: numpy.ndarray, byte_ord_str: str, check_type: bool=True) -> numpy.ndarray:

        """
        Load the requested variable and return as a NumPy array.
        args...
            var_name: variable name
            var_data: variable data (string of bytes)
            dim_sels: dimension selections (see get_request_url)
            byte_ord_str: byte order string: '<' for little endian, '>' for big endian
        kwargs...
            check_type: check header string contains expected variable type
        """

        dims = False
        if dim_sels.shape[1] == 3:
            dims = True
        #  Otherwise size is [1,1] and contains number of elements in
        #  dimensionless data

        if dims:
            boffs = 8
            #  Byte offset for the two occurrences of the number of
            #  elements (int32)
        else:
            boffs = 4
            #  Offset for dimensionless data

        #  Find the header portion of the byte stream
        # (until the 'Data' identifier)

        data_id = 'Data:\n'.encode('utf-8')
        id_len = len(data_id)
        data_len = len(var_data)

        null_ret = numpy.empty(0)

        i = 0
        while (i < data_len - id_len - 1) and (var_data[i:i + id_len] != data_id):
            i += 1
        if var_data[i:i + id_len] != data_id:
            print('VariableLoader: Data start identifier not found, stopping.')
            return null_ret

        data_start_ind = i + id_len
        data_start_ind += boffs

        # Check the var_data byte stream contains the correct variable type
        hdr_str = var_data[:data_start_ind - boffs].decode('utf-8')
        var_type = self.dds[var_name][0]
        if check_type and (var_type not in hdr_str):
            print('VariableLoader: Variable type in requested data header does not match DDS, stopping.')
            return null_ret

        #  Check the var_data byte stream contains the correct
        # variable dimensions

        dim_str = var_name
        if dims:
            dim_str, num_els = self.get_dim_str(dim_sels, var_name)
        else:
            num_els = dim_sels[0, 0]  # Number of elements to retrieve

        if dim_str not in hdr_str:
            print('VariableLoader: Variable dimensions in requested data header do not match DDS, stopping.')
            return null_ret

        # All OK - load the variable

        np_type = Definitions.atomics[var_type]
        data_type = np_type

        if len(byte_ord_str) > 0:
            data_type = np_type.newbyteorder(byte_ord_str)

        lvar = numpy.frombuffer(var_data, dtype=data_type, count=int(numpy.prod(num_els)), offset=data_start_ind)
        if dims:
            var = lvar.reshape(tuple(num_els))
        else:
            var = lvar

        return var

    def get_dim_str(self, dim_sels: numpy.ndarray, var_name: str) -> tuple:

        """
        Get the dimension selection string and number of elements.
        args...
            dim_sels: dimension selections (see get_request_url)
            var_name: variable name
        """

        assoc_names = self.dds[var_name][2]
        dim_str = var_name
        num_els = []
        for idim in range(dim_sels.shape[0]):
            count = 1
            ind = dim_sels[idim, 0]
            while ind < dim_sels[idim, 2]:
                ind += dim_sels[idim, 1]
                count += 1
            if ind > dim_sels[idim, 2]:
                count -= 1
            num_els.append(count)
            dim_str += '[' + assoc_names[idim] + ' = ' + \
                       str(num_els[idim]) + ']'

        return dim_str, num_els



"""
Example use of sodapclient.
To run this example at the command line enter: python3 Example.py
"""

import numpy

from sodapclient.Handler import Handler

def run_example():

    # Define the URL (one of the datasets available at the official
    # OpenDAP test server):
    url = 'http://test.opendap.org/opendap/hyrax/data/nc/sst.mnmean.nc.gz.html'

    # Initialise handler, which will also get the DDS & DAS
    handler = Handler(url, proxy_file_name='proxyserver.txt')

    # We'll access the SST data over the last 2 times, the last 3 lats and the
    # last 4 longs available...

    # Set up the arrays for the required dimensions, shape 1x3 for map variables
    # and nx3 for data variables with n dimensions
    time_dims = numpy.array([[1800, 10, 1810]])  # Time indices 1800 and 1810
    lat_dims = numpy.array([[70, 2, 74]])  # Latitude indices 70, 72 and 74
    lon_dims = numpy.array([[80, 1, 83]])  # Longitude indices 80 to 83
    sst_dims = numpy.array([time_dims[0], lat_dims[0], lon_dims[0]])

    byte_order = '>'  # Big Endian

    # Get the data and the dimension variables

    handler.get_variable('time', time_dims, byte_order)
    handler.get_variable('lat', lat_dims, byte_order)
    handler.get_variable('lon', lon_dims, byte_order)
    handler.get_variable('sst', sst_dims, byte_order)

    handler.print()  # Print the DDS, DAS and data to screen

    # Then get access to any variable from the handler's variables dictionary, e.g.

    sst = handler.variables['sst']
    print('\nSST...\n', sst)


if __name__ == '__main__':
    run_example()



"""
ProxyDict class definition
"""


class ProxyDict:

    """
    A class to provide proxy server details.
    Reads in the proxy details from a text file and returns a dictionary of
    proxy strings as required for class urllib.request.ProxyHandler
    Example proxy server text file contained in Examples package.
    """

    def __init__(self, proxy_file_name: str) -> None:

        """
        args...
            proxy_file_name: path to text file containing proxy details
        """

        self.valid_proxy = True
        proxy_config = {}  # This is a convenience dictionary

        with open(proxy_file_name, 'rt') as file:
            for line in file:
                # Assumes newline at end of all lines
                srcdata = line[:-1].split(':')
                if (len(srcdata) > 1) and \
                   ('<' not in srcdata[1]) and ('>' not in srcdata[1]):
                    proxy_config[srcdata[0]] = srcdata[1].strip()  # No spaces

        if ('server' not in proxy_config) or \
           ('port' not in proxy_config) or \
           ('methods' not in proxy_config):
            self.valid_proxy = False
        else:
            # Will read in string so convert to list
            proxy_config['methods'] = proxy_config['methods'].split(',')
            self.proxy_config = proxy_config

    def get_dict(self) -> dict:

        """
        Returns a dictionary containing the proxy data
        as required by urllib.request.ProxyHandler
        """

        proxy_strs = {}

        for method in self.proxy_config['methods']:
            if ('user' in self.proxy_config) and \
               ('password' in self.proxy_config):
                proxy_strs[method] = method + '://' + \
                    self.proxy_config['user'] + ':' + \
                    self.proxy_config['password'] + '@' + \
                    self.proxy_config['server'] + ':' + \
                    str(self.proxy_config['port']) + '/'
            else:
                proxy_strs[method] = method + '://' + \
                    self.proxy_config['server'] + ':' + \
                    str(self.proxy_config['port']) + '/'

        return proxy_strs



"""
TestProxyDict class definition
"""

import unittest
from os import remove
from sodapclient.ProxyDict import ProxyDict


class TestProxyDict(unittest.TestCase):

    """
    Test class for ProxyDict class.
    """

    def setUp(self):

        # Define the text proxy details
        self.proxy_config = {'user': 'Me',
                             'password': 'abracadabra',
                             'server': 'magic.co.uk',
                             'port': '1234',
                             'methods': ['http', 'https', 'ftp', 'socks']}

        # Write the test proxy details to the file
        self.proxy_file_name = 'ProxyDictTests.txt'
        file = open(self.proxy_file_name, 'wt')
        file.write('user' + ':' + self.proxy_config['user'] + '\n')
        file.write('password' + ':' + self.proxy_config['password'] + '\n')
        file.write('server' + ':' + self.proxy_config['server'] + '\n')
        file.write('port' + ':' + str(self.proxy_config['port']) + '\n')
        file.write('methods' + ':')
        # Loop is a bit clunky but needed to avoid trailing comma...
        for method in range(len(self.proxy_config['methods'])):
            if method < len(self.proxy_config['methods']) - 1:
                file.write(self.proxy_config['methods'][method] + ',')
            else:
                file.write(self.proxy_config['methods'][method] + '\n')
        file.close()

    def tearDown(self):
        remove(self.proxy_file_name)

    def test_constructor(self):

        """
        Test the ProxyDict constructor.
        """

        pdict = ProxyDict(self.proxy_file_name)
        self.assertEqual(self.proxy_config, pdict.proxy_config)

    def test_get_dict(self):

        """
        Define the dictionary which should be returned and test for it.
        """

        proxy_test_dict = {'http': 'http://Me:abracadabra@magic.co.uk:1234/',
                           'https': 'https://Me:abracadabra@magic.co.uk:1234/',
                           'ftp': 'ftp://Me:abracadabra@magic.co.uk:1234/',
                           'socks': 'socks://Me:abracadabra@magic.co.uk:1234/'}
        pdict = ProxyDict(self.proxy_file_name)
        tdict = pdict.get_dict()
        self.assertEqual(proxy_test_dict, tdict)


if __name__ == "__main__":
    unittest.main()
